# From Imports
from imohash import hashfile as hf
from io import TextIOWrapper
from os import path as osPath
from sarge import capture_both
from sys import exit
from typing import List, Callable, Any, Union, Tuple
from xxhash import xxh32, xxh64

# Local Imports
from nanite import nanite

pf: Callable[..., bool] = osPath.isfile
pd: Callable[..., bool] = osPath.isdir
fullpath: Callable[..., str] = nanite().fullpath
enum: Callable[..., Any] = enumerate


class Error(Exception):
    pass


class Shrug(Error):
    pass


class noCheck(Error):
    pass


class noDir(Error):
    pass


class checks:
    def __init__(
        self,
        source: Union[str, None] = None,
        hashFile: Union[str, None] = None,
        dryRun: bool = False,
    ):
        """
			Variable "source" takes a single string path
		"""
        self.source: Union[str, None] = fullpath(
            source
        ) if source else None
        self.hashFile: Union[str, None] = fullpath(
            hashFile
        ) if hashFile else None
        self.dryRun: bool = dryRun
        self.hashList: List[str] = []
        self.vcDict: Dict[str, Tuple[str]] = {
            "hg": (
                ".hg",
                f"hg st -R {self.source}",
                "mercurial",
            ),
            "git": (
                ".git",
                f"git -C {self.source} status --porcelain",
                "git",
            ),
        }

    def _compareAndWriteToFile(self) -> bool:
        fileLen: int = 0
        with open(self.hashFile, "r") as file:
            for line in file:
                fileLen = fileLen + 1

        if len(self.hashList) != fileLen:
            if self.dryRun:
                return True
            else:
                with open(self.hashFile, "w") as file:
                    for _hash in self.hashList:
                        file.write(_hash + "\n")
                return True
        else:
            with open(self.hashFile, "r+") as file:
                for index, line in enum(file):
                    line = line.rstrip()
                    if self.hashList[index] != line:
                        if self.dryRun:
                            return True
                        else:
                            file.seek(0)
                            for _hash in self.hashList:
                                file.write(_hash + "\n")
                            return True
            return False

    def fd(
        self, time: str, ignoreFile: str, followSymlinks: bool
    ) -> bool:
        """
			Returns a False if the "fd" shell command returns an empty list, else a True if it returns a list of changed files

			Variable "fdCh" uses the output from the shell command, using the "time" and "ignoreFile" arguments, as well as the class' "source" argument, to procure a list of changed files within the given time period (in miutes). The unformatted output is "utf-8" decoded, then split into a list.
		"""
        if pd(self.source):
            ignoreFile = fullpath(ignoreFile)
            cmd: str = (
                "fd ." + " --follow"
                if followSymlinks
                else ""
                + f" --full-path --hidden --no-ignore-vcs --changed-within {time}min --ignore-file {ignoreFile} {self.source}"
            )
            fdSt = capture_both(cmd)
            fdStReturnCodes: List[int] = fdSt.returncodes
            if any(fdStReturnCodes):
                print(
                    f"Sorry; something happened! This was the error:\n"
                )
                for line in TextIOWrapper(fdSt.stderr):
                    print(line.rstrip())
                raise Shrug()
            else:
                for line in TextIOWrapper(fdSt.stdout):
                    return bool(line.rstrip())
        else:
            raise noDir(
                f"Sorry! {self.source} doesn't seem to exist!"
            )

    def vc(self, _vc: str) -> bool:
        """
            Temp
        """
        repoExt, command, vcName = self.vcDict[_vc]
        if pd(fullpath(self.source, repoExt)):
            st = capture_both(command)
            stReturnCodes: List[int] = st.returncodes
            if any(stReturnCodes):
                print(
                    f"Sorry; something happened! This was the error:\n"
                )
                for line in TextIOWrapper(st.stderr):
                    print(line.rstrip())
                raise Shrug()
            else:
                self.hashList = [
                    line.rstrip()
                    for line in TextIOWrapper(st.stdout)
                ]
                return self._compareAndWriteToFile()
        else:
            raise noDir(
                f"Sorry! {self.source} doesn't seem to be a {vcName} repository!"
            )

    def hash(
        self,
        hashType: str,
        includeFile: str,
        chunkSize: int,
        x64: bool = False,
    ) -> bool:

        hashes: Tuple[str] = ("xxhash", "imohash")
        if hashType not in hashes:
            raise noCheck(
                f'Sorry! There is no hash type "{hashType}"!'
            )

        includeFile: str = fullpath(includeFile)

        def xxhashfile():
            h = xxh64() if x64 else xxh32()
            with open(line, "rb") as file:
                # chunkSize is in bits
                for chunk in iter(
                    lambda: file.read(chunkSize), b""
                ):
                    h.update(chunk)
            return h.hexdigest()

        with open(includeFile) as file:
            for line in file:
                line = line.rstrip()
                if pf(line):
                    if hashType == "xxhash":
                        self.hashList.append(xxhashfile())
                    elif hashType == "imohash":
                        self.hashList.append(
                            hf(line, hexdigest=True)
                        )
                elif pd(line):
                    self.hashList.append(line.rstrip())
        return self._compareAndWriteToFile()
