# Imports
# import importlib
# import site
import glob
import os

# From Imports
from itertools import chain, combinations
from subprocess import run, PIPE
from sys import exit, executable
from typing import Dict, List, Tuple, Any, Callable, Generator

# Local Imports
from ignores import ignores
from nanite import nanite

fullpath: Callable[..., str] = nanite().fullpath


class Error(Exception):
    pass


class GITNotSet(Error):
    pass


class shrug(Error):
    pass


class etc:
    def __init__(self):
        """
            Random functions not related to anything else in the package.
        """
        pass

    def GITInfo(self) -> Generator[str, None, None]:
        """
            Returns a list of GIT environmental values
        """
        try:
            yield os.environ["GIT_AUTHOR_NAME"]
            yield os.environ["GIT_AUTHOR_EMAIL"]
            yield os.environ["GIT_COMMITTER_NAME"]
            yield os.environ["GIT_COMMITTER_EMAIL"]
        except:
            raise GITNotSet(
                'Sorry, something happened; check your "GIT" variables.'
            )

    def powerset(
        iterable: List[Any]
    ) -> Generator[Any, None, None]:
        """
            Credits: https://docs.python.org/3/library/itertools.html#itertools-recipes
            Modified by me
            powerset([1,2,3]) --> [(), (1,) (2,) (3,) (1, 2) (1, 3) (2, 3) (1, 2, 3)]
        """
        s: Tuple[List[Any]] = tuple(iterable)
        yield chain.from_iterable(
            combinations(s, r) for r in range(len(s) + 1)
        )

    def genInclFile(
        ignoreList: List[str],
        path: str,
        includeFile: str,
        followSymlinks: bool,
    ):
        """
            Generate the list of included directories within the given "path" variable, using the "fd" command, ignoring what's in the "ignoreFile" variable, and put it into the file in the "includeFile" variable
        """
        _ = chain(
            glob.iglob(
                f'{path}/**/*{ignores(ignoreList).glob() if ignoreList else ""}',
                recursive=True,
            ),
            glob.iglob(
                f"{path}/**/.*{ignores(ignoreList).glob()}",
                recursive=True,
            ),
        )
        try:
            with open(fullpath(includeFile), "w+") as file:
                for fd in _:
                    file.write(
                        fullpath(fd) + "\n"
                        if followSymlinks
                        else fullpath(fd) + "\n"
                    )
        except Exception as e:
            raise shrug(
                "Sorry; something happened! This was the error:\n{e}"
            )

    # Fix this!
    # def installImport(module: str):
    #     """
    #         Credits: https://stackoverflow.com/questions/12332975/installing-python-module-within-code/50255019#50255019
    #                  https://stackoverflow.com/a/24773951/10827766
    #         Modified by me
    #     """
    #     try:
    #         importlib.import_module(module)
    #     except ImportError:
    #         run([executable, "-m", "pip", "install", module])
    #     except Exception as e:
    #         raise shrug(f"Something happened! This was the error:\n{e}")
    #     finally:
    #         globals()[module] = importlib.import_module(module)
