#!/usr/bin/env python3

if __name__ == "__main__":
    # Imports
    import click
    import os
    import schedule
    import sh
    import time

    # From Imports
    from dataclasses import dataclass, field
    from loguru import logger
    from typing import List, Any, Callable

    # Local Imports
    from ccc import naniteOp, naniteComm
    from nanite import nanite

    @click.group(
        invoke_without_command=True, no_args_is_help=True
    )
    @logger.catch
    def main():
        pass

    @click.command(cls=naniteComm, no_args_is_help=True)
    @click.option(
        "-p",
        "--path",
        required=True,
        type=click.Path(),
        help="The directory with the file to tail",
    )
    @click.option(
        "-t",
        "--time-fv",
        required=True,
        type=(str, int),
        help='The time format and value; the format can be either "hour" or "min", while the value can be between 0 and 60 exclusive.',
    )
    @logger.catch
    def tail(path, time_fv):

        if os.name == "nt":
            logger.error(
                "Sorry! This subcommand cannot be used on windows!"
            )
            exit(1)

        dfl: Callable[..., Any] = field(default_factory=list)

        @dataclass
        class data:
            previous_tail: List[str] = dfl
            current_tail: List[str] = dfl

        d: type = data()
        d.previous_tail = []

        if "hour" in time_fv:
            schedule_func = schedule.every(time_fv[1]).hours.do
        elif "min" in time_fv:
            if not 0 < time_fv[1] < 60:
                logger.error("Sorry! That is an invalid value!")
                exit(1)
            else:
                schedule_func = schedule.every(
                    time_fv[1]
                ).minutes.do
        else:
            logger.error(
                "Sorry! That is not a valid time format!"
            )
            exit(1)

        def inner() -> None:
            resolved_path = nanite().fullpath(path, check=True)
            list_of_files = (
                f"{resolved_path}/{_}"
                for _ in os.listdir(resolved_path)
            )
            latest_file = max(
                list_of_files, key=os.path.getctime
            )
            d.current_tail = [
                line
                for line in sh.tail(
                    latest_file, _err_to_out=True, _iter=True
                )
            ]
            if d.current_tail != d.previous_tail:
                for line in d.current_tail:
                    click.echo(line)
                d.previous_tail[:] = d.current_tail[:]
                click.secho("\n~//~\n", fg="red")

        inner()
        schedule_func(inner)

        while True:
            schedule.run_pending()
            time.sleep(1)

    main.add_command(tail)

    main()

else:
    # Local Imports
    from ccc import naniteOp, naniteComm
    from checks import checks
    from decorators import decorators
    from etc import etc
    from gen_tests import tests
    from ignores import ignores
    from joins import joins
    from nanite import nanite

    zoom = decorators.zoom
    savitar = decorators.savitar
    godspeed = decorators.godspeed
